@@ -5,8 +5,8 @@
 #include "clock-typebuiltins.h"
 
 
-/* enumerations from "../../applets/clock/clock.h" */
-#include "../../applets/clock/clock.h"
+/* enumerations from "../../applets/clock/clock-utils.h" */
+#include "../../applets/clock/clock-utils.h"
 static const GEnumValue _clock_format_values[] = {
   { CLOCK_FORMAT_INVALID, "CLOCK_FORMAT_INVALID", "invalid" },
   { CLOCK_FORMAT_12, "CLOCK_FORMAT_12", "12" },
