@@ -70,7 +70,7 @@ struct _PanelBackgroundMonitor {
 	gboolean   display_grabbed;
 };
 
-G_DEFINE_TYPE (PanelBackgroundMonitor, panel_background_monitor, G_TYPE_OBJECT);
+G_DEFINE_TYPE (PanelBackgroundMonitor, panel_background_monitor, G_TYPE_OBJECT)
 
 static PanelBackgroundMonitor **global_background_monitors = NULL;
 
@@ -124,7 +124,7 @@ panel_background_monitor_init (PanelBackgroundMonitor *monitor)
 	monitor->gdkwindow = NULL;
 	monitor->xwindow   = None;
 
-	monitor->gdkatom = gdk_atom_intern ("_XROOTPMAP_ID", FALSE);
+	monitor->gdkatom = gdk_atom_intern_static_string ("_XROOTPMAP_ID");
 	monitor->xatom   = gdk_x11_atom_to_xatom (monitor->gdkatom);
 
 	monitor->gdkpixmap = NULL;
