@@ -35,9 +35,9 @@
 #include "panel-icon-names.h"
 #include "panel-stock-icons.h"
 
-GdkFilterReturn popup_filter (GdkXEvent *gdk_xevent,
-			      GdkEvent  *event,
-			      GtkWidget *popup);
+static GdkFilterReturn popup_filter (GdkXEvent *gdk_xevent,
+				     GdkEvent  *event,
+				     GtkWidget *popup);
 
 static Atom wm_state_atom = None;
 
@@ -205,9 +205,9 @@ kill_window_question (gpointer window)
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  _("If you choose to force an application "
-						  "to exit, any open documents in it will "
-						  "be lost."));
- 
+						  "to exit, unsaved changes in any open documents "
+						  "in it might get lost."));
+
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL,
 				GTK_RESPONSE_CANCEL,
@@ -248,7 +248,7 @@ handle_button_press_event (GtkWidget *popup,
 	}
 }
 
-GdkFilterReturn
+static GdkFilterReturn
 popup_filter (GdkXEvent *gdk_xevent,
 	      GdkEvent  *event,
 	      GtkWidget *popup)
