@@ -40,15 +40,17 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade-xml.h>
-#include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-entry.h>
-#include <libgnomeui/gnome-url.h>
 #include <gconf/gconf-client.h>
 #include <gmenu-tree.h>
 
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnome/gnome-desktop-utils.h>
+
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-gtk.h>
 #include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-show.h>
 
 #include "nothing.h"
 #include "panel-gconf.h"
@@ -69,8 +71,7 @@ typedef struct {
 	GtkWidget        *main_box;
 	GtkWidget        *program_list_box;
 
-	GtkWidget        *gnome_entry;
-	GtkWidget        *gtk_entry;
+	GtkWidget        *combobox;
 	GtkWidget        *pixmap;
 	GtkWidget        *run_button;
 	GtkWidget        *file_button;
@@ -116,6 +117,74 @@ static PanelRunDialog *static_dialog = NULL;
 
 static void panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog);
 
+#define PANEL_RUN_HISTORY_KEY "/apps/gnome-settings/gnome-panel/history-gnome-run"
+#define PANEL_RUN_MAX_HISTORY 10
+
+static GtkTreeModel *
+_panel_run_get_recent_programs_list (void)
+{
+	GtkListStore *list;
+	GSList       *gconf_items;
+	GSList       *command;
+	int           i = 0;
+
+	list = gtk_list_store_new (1, G_TYPE_STRING);
+
+	gconf_items = gconf_client_get_list (panel_gconf_get_client (),
+					     PANEL_RUN_HISTORY_KEY,
+					     GCONF_VALUE_STRING, NULL);
+
+	for (command = gconf_items;
+	     command && i < PANEL_RUN_MAX_HISTORY;
+	     command = command->next) {
+		GtkTreeIter iter;
+		gtk_list_store_prepend (list, &iter);
+		gtk_list_store_set (list, &iter, 0, command->data, -1);
+		i++;
+	}
+
+	g_slist_free (gconf_items);
+
+	return GTK_TREE_MODEL (list);
+}
+
+static void
+_panel_run_save_recent_programs_list (GtkComboBoxEntry *entry,
+				      char             *lastcommand)
+{
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	GSList       *gconf_items = NULL;
+	int           i = 0;
+
+	gconf_items = g_slist_prepend (gconf_items, lastcommand);
+	i++;
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
+
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		char *command;
+
+		do {
+			gtk_tree_model_get (model, &iter, 0, &command, -1);
+
+			if (strcmp (command, lastcommand) == 0)
+				continue;
+
+			gconf_items = g_slist_prepend (gconf_items, command);
+			i++;
+		} while (gtk_tree_model_iter_next (model, &iter) &&
+			 i < PANEL_RUN_MAX_HISTORY);
+	}
+
+	gconf_client_set_list (panel_gconf_get_client (),
+			       PANEL_RUN_HISTORY_KEY,
+			       GCONF_VALUE_STRING, gconf_items,
+			       NULL);
+
+	g_slist_free (gconf_items);
+}
+
 static void
 panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
@@ -244,29 +313,27 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 }
 
 static gboolean
-command_is_executable (const char *command)
+command_is_executable (const char   *command,
+		       int          *argcp,
+		       char       ***argvp)
 {
 	gboolean   result;
 	char     **argv;
 	char      *path;
 	int        argc;
-	
-	path = g_filename_from_uri (command, NULL, NULL);
-	if (!path)
-		path = g_strdup (command);
-	
-	result = g_shell_parse_argv (path, &argc, &argv, NULL);
-	g_free (path);
-	
+
+	result = g_shell_parse_argv (command, &argc, &argv, NULL);
+
 	if (!result)
 		return FALSE;
-	
-	path = g_find_program_in_path (argv [0]);
-	g_strfreev (argv);
-			
-	if (!path)
+
+	path = g_find_program_in_path (argv[0]);
+
+	if (!path) {
+		g_strfreev (argv);
 		return FALSE;
-	
+	}
+
 	/* If we pass an absolute path to g_find_program it just returns
 	 * that absolute path without checking if it is executable. Also
 	 * make sure its a regular file so we don't try to launch
@@ -275,18 +342,24 @@ command_is_executable (const char *command)
 	if (!g_file_test (path, G_FILE_TEST_IS_EXECUTABLE) ||
 	    !g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
 		g_free (path);
+		g_strfreev (argv);
 		return FALSE;
 	}
-	
+
 	g_free (path);
-	
+
+	if (argcp)
+		*argcp = argc;
+	if (argvp)
+		*argvp = argv;
+
 	return TRUE;
 }
 
 static gboolean
 panel_run_dialog_launch_command (PanelRunDialog *dialog,
 				 const char     *command,
-				 const char     *escaped)
+				 const char     *locale_command)
 {
 	GdkScreen  *screen;
 	gboolean    result;	
@@ -294,19 +367,13 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	char      **argv;
 	int         argc;
 	
-	if (!command_is_executable (command))
+	if (!command_is_executable (locale_command, &argc, &argv))
 		return FALSE;
 
-	argc = 3;
-	argv = g_new0 (char *, 4);
-	argv [0] = gnome_util_user_shell ();
-	argv [1] = g_strdup ("-c");
-	argv [2] = g_strdup (command);
-	
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));	
 		
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)))
-		gnome_prepend_terminal_to_vector (&argc, &argv);
+		gnome_desktop_prepend_terminal_to_vector (&argc, &argv);
 		   
 	result = gdk_spawn_on_screen (screen,
 				      NULL, /* working directory */
@@ -321,8 +388,8 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	if (!result) {
 		char *primary;
 
-		primary = g_strdup_printf (_("Could not run command '%s'"),
-					   escaped);
+		primary = g_markup_printf_escaped (_("Could not run command '%s'"),
+						   command);
 		panel_error_dialog (GTK_WINDOW (dialog->run_dialog), NULL,
 				    "cannot_spawn_command", TRUE,
 				    primary, error->message);
@@ -336,48 +403,16 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	return result;
 }
 
-static gboolean
-panel_run_dialog_show_url (PanelRunDialog *dialog,
-	                   const char     *url,
-	                   const char     *escaped)
-{
-	GError    *error = NULL;
-	GdkScreen *screen;
-	
-	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
-
-	gnome_url_show_on_screen (url, screen, &error);
-	if (error) {
-		char *primary;
-
-		primary = g_strdup_printf (_("Could not open location '%s'"),
-					   escaped);
-		panel_error_dialog (GTK_WINDOW (dialog->run_dialog), NULL,
-				    "cannot_show_url", TRUE,
-				    primary, error->message);
-		g_free (primary);
-
-		g_error_free (error);
-		return FALSE;
-	}
-	
-	return TRUE;
-}
-
 static void
 panel_run_dialog_execute (PanelRunDialog *dialog)
 {
-	GdkScreen *screen;
-	GError    *error;
-	gboolean   result;
-	char      *command;
-	char      *escaped;
-	GFile     *file;
-	char      *disk, *url;
-	char      *scheme;	
+	GError   *error;
+	gboolean  result;
+	char     *command;
+	char     *disk;
+	char     *scheme;	
 	
-	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));	
-	command = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry)));
+	command = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox));
 	command = g_strchug (command);
 
 	if (!command || !command [0]) {
@@ -417,31 +452,34 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 		return;
 	}
 
-	file = panel_util_get_file_optional_homedir (command);
-	url = g_file_get_uri (file);
-	scheme = g_file_get_uri_scheme (file);
-	g_object_unref (file);
-
-	escaped = g_markup_escape_text (url, -1);
 	result = FALSE;
 	
-	if (!g_ascii_strcasecmp (scheme, "http") ||
-	    !g_ascii_strcasecmp (scheme, "file"))
-		/* If this returns an http or file url, the url might refer to
-		 * a command that is somewhere in the path or an executable
-		 * file. So try executing it before displaying it. We execute
-		 * the command in the user's shell so that it can do all the
-		 * parameter expansion and other magic for us. */
-		result = panel_run_dialog_launch_command (dialog, disk, escaped);
+	scheme = g_uri_parse_scheme (disk);
+	/* if it's an absolute path or not a URI, it's possibly an executable,
+	 * so try it before displaying it */
+	if (g_path_is_absolute (disk) || !scheme)
+		result = panel_run_dialog_launch_command (dialog, command, disk);
 	
-	if (!result)
-		result = panel_run_dialog_show_url (dialog, url, escaped);
+	if (!result) {
+		GFile     *file;
+		char      *uri;
+		GdkScreen *screen;
+		
+		file = panel_util_get_file_optional_homedir (command);
+		uri = g_file_get_uri (file);
+		g_object_unref (file);
+
+		screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
+		result = panel_show_uri (screen, uri,
+					 gtk_get_current_event_time (), NULL);
+
+		g_free (uri);
+	}
 		
 	if (result) {
 		/* only save working commands in history */
-		gnome_entry_prepend_history (GNOME_ENTRY (dialog->gnome_entry),
-					     TRUE, /* save item in history */
-					     command);
+		_panel_run_save_recent_programs_list
+			(GTK_COMBO_BOX_ENTRY (dialog->combobox), command);
 		
 		/* only close the dialog if we successfully showed or launched
 		 * something */
@@ -450,8 +488,6 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 
 	g_free (command);
 	g_free (disk);
-	g_free (url);
-	g_free (escaped);
 	g_free (scheme);
 }
 
@@ -472,7 +508,7 @@ panel_run_dialog_response (PanelRunDialog *dialog,
 		break;
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (run_dialog)),
-				 "user-guide.xml", "gospanel-23");
+				 "user-guide", "gospanel-23", NULL);
 		break;
 	default:
 		break;
@@ -502,23 +538,25 @@ static void
 panel_run_dialog_append_file_utf8 (PanelRunDialog *dialog,
 				   const char     *file)
 {
-	const char *text;
+	char       *text;
 	char       *quoted, *temp;
+	GtkWidget  *entry;
 	
 	/* Don't allow filenames beginning with '-' */
 	if (!file || !file[0] || file[0] == '-')
 		return;
 	
 	quoted = quote_string (file);
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
-	
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+	text = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox));
 	if (text && text [0]) {
 		temp = g_strconcat (text, " ", quoted, NULL);
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), temp);
+		gtk_entry_set_text (GTK_ENTRY (entry), temp);
 		g_free (temp);
 	} else
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry), quoted);
+		gtk_entry_set_text (GTK_ENTRY (entry), quoted);
 	
+	g_free (text);
 	g_free (quoted);
 }
 
@@ -608,17 +646,13 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	GtkTreeIter   iter;
 	GtkTreeModel *model;
 	GtkTreePath  *path;
-	const char   *text;
+	char         *text;
 	char         *found_icon;
 	char         *found_name;
 	gboolean      fuzzy;
 	
 	model = GTK_TREE_MODEL (dialog->program_list_store);
 	path = gtk_tree_path_new_first ();
-	text = sure_string (gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry)));
-	found_icon = NULL;
-	found_name = NULL;
-	fuzzy = FALSE;
 	
 	if (!path || !gtk_tree_model_get_iter (model, &iter, path)) {
 		if (path)
@@ -630,6 +664,11 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 		return FALSE;
 	}
 
+	text = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox));
+	found_icon = NULL;
+	found_name = NULL;
+	fuzzy = FALSE;
+
 	do {
 		char *exec = NULL;
 		char *icon = NULL;
@@ -655,9 +694,9 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 					    &iter,
 					    COLUMN_VISIBLE, TRUE,
 					    -1);
-		} else if (panel_util_utf8_strstrcase (exec, text) != NULL ||
-			   panel_util_utf8_strstrcase (name, text) != NULL ||
-			   panel_util_utf8_strstrcase (comment, text) != NULL) {
+		} else if (panel_g_utf8_strstrcase (exec, text) != NULL ||
+			   panel_g_utf8_strstrcase (name, text) != NULL ||
+			   panel_g_utf8_strstrcase (comment, text) != NULL) {
 			gtk_list_store_set (dialog->program_list_store,
 					    &iter,
 					    COLUMN_VISIBLE, TRUE,
@@ -687,6 +726,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	//FIXME update dialog->program_label
 
 	g_free (found_icon);
+	g_free (text);
 	
 	g_free (dialog->item_name);
 	dialog->item_name = found_name;
@@ -993,7 +1033,8 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	char         *path, *stripped;
 	gboolean      terminal;
 	GKeyFile     *key_file;
-		
+	GtkWidget    *entry;
+
 	if (!gtk_tree_selection_get_selected (selection, &filter_model,
 					      &filter_iter))
 		return;
@@ -1026,21 +1067,20 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	if (dialog->item_name)
 		g_free (dialog->item_name);
 	dialog->item_name = NULL;
-	
+
 	/* Order is important here. We have to set the text first so that the
 	 * drag source is enabled, otherwise the drag icon can't be set by
 	 * panel_run_dialog_set_icon.
 	 */
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
 	temp = panel_key_file_get_string (key_file, "Exec");
 	if (temp) {
 		stripped = remove_parameters (temp);
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry),
-				    stripped);
+		gtk_entry_set_text (GTK_ENTRY (entry), stripped);
 		g_free (stripped);
 	} else {
 		temp = panel_key_file_get_string (key_file, "URL");
-		gtk_entry_set_text (GTK_ENTRY (dialog->gtk_entry),
-				    sure_string (temp));
+		gtk_entry_set_text (GTK_ENTRY (entry), sure_string (temp));
 	}
 	g_free (temp);
 
@@ -1082,14 +1122,14 @@ program_list_selection_activated (GtkTreeView       *view,
 
 static void
 panel_run_dialog_setup_program_list (PanelRunDialog *dialog,
-				     GladeXML       *gui)
+				     GtkBuilder     *gui)
 {
 	GtkTreeSelection *selection;
 	
-	dialog->program_list = glade_xml_get_widget (gui, "program_list");
-	dialog->program_list_box = glade_xml_get_widget (gui, "program_list_box");
-	dialog->program_label = glade_xml_get_widget (gui, "program_label");
-	dialog->main_box = glade_xml_get_widget (gui, "main_box");
+	dialog->program_list = PANEL_GTK_BUILDER_GET (gui, "program_list");
+	dialog->program_list_box = PANEL_GTK_BUILDER_GET (gui, "program_list_box");
+	dialog->program_label = PANEL_GTK_BUILDER_GET (gui, "program_label");
+	dialog->main_box = PANEL_GTK_BUILDER_GET (gui, "main_box");
 	
 	/* Ref the box so it doesn't get destroyed when it is
 	 * removed from the visible area of the dialog box.
@@ -1125,7 +1165,7 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 					      dialog->list_expander);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
-                gtk_widget_grab_focus (dialog->gtk_entry);
+                gtk_widget_grab_focus (dialog->combobox);
 		
 	} else if (show_list) {
 		gtk_window_resize (GTK_WINDOW (dialog->run_dialog), 100, 300);
@@ -1134,7 +1174,7 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 		
         } else if (!show_list) {
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
-                gtk_widget_grab_focus (dialog->gtk_entry);
+                gtk_widget_grab_focus (dialog->combobox);
         }
 }
 
@@ -1157,12 +1197,12 @@ list_expander_toggled (GtkExpander    *expander,
 
 static void
 panel_run_dialog_setup_list_expander (PanelRunDialog *dialog,
-				      GladeXML       *gui)
+				      GtkBuilder     *gui)
 {
 	GConfClient *client;
 	const char *key;
 	
-	dialog->list_expander = glade_xml_get_widget (gui, "list_expander");
+	dialog->list_expander = PANEL_GTK_BUILDER_GET (gui, "list_expander");
 
 	if (panel_profile_get_enable_program_list ()) {
 		gtk_expander_set_expanded (GTK_EXPANDER (dialog->list_expander),
@@ -1203,7 +1243,7 @@ file_button_browse_response (GtkWidget      *chooser,
 
 	gtk_widget_destroy (chooser);
  
- 	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_grab_focus (dialog->combobox);
 }
 
 static void
@@ -1233,9 +1273,9 @@ file_button_clicked (GtkButton      *button,
 
 static void
 panel_run_dialog_setup_file_button (PanelRunDialog *dialog,
-				    GladeXML       *gui)
+				    GtkBuilder     *gui)
 {
-	dialog->file_button = glade_xml_get_widget (gui, "file_button");
+	dialog->file_button = PANEL_GTK_BUILDER_GET (gui, "file_button");
 		
         g_signal_connect (dialog->file_button, "clicked",
 			  G_CALLBACK (file_button_clicked),
@@ -1527,10 +1567,10 @@ entry_event (GtkEditable    *entry,
 
 			prefix = g_strconcat (temp, nprefix, NULL);
 
-			g_signal_handler_block (dialog->gtk_entry,
+			g_signal_handler_block (dialog->combobox,
 						dialog->changed_id);
 			gtk_editable_delete_text (entry, 0, -1);
-			g_signal_handler_unblock (dialog->gtk_entry,
+			g_signal_handler_unblock (dialog->combobox,
 						  dialog->changed_id);
 
 			gtk_editable_insert_text (entry,
@@ -1556,14 +1596,14 @@ entry_event (GtkEditable    *entry,
 }
 
 static void
-entry_changed (GtkEntry       *entry,
-	       PanelRunDialog *dialog)
+combobox_changed (GtkComboBox    *combobox,
+		  PanelRunDialog *dialog)
 {
-	const char *text;
-	const char *start;
-	char       *msg;
+	char *text;
+	char *start;
+	char *msg;
 
-	text = gtk_entry_get_text (entry);
+	text = gtk_combo_box_get_active_text (combobox);
 
 	start = text;
 	while (*start != '\0' && g_ascii_isspace (*start))
@@ -1583,6 +1623,8 @@ entry_changed (GtkEntry       *entry,
 
 	/* desensitize run button if no text entered */
 	if (!start || !start [0]) {
+		g_free (text);
+
 		gtk_widget_set_sensitive (dialog->run_button, FALSE);
 		gtk_drag_source_unset (dialog->run_dialog);
 
@@ -1640,6 +1682,8 @@ entry_changed (GtkEntry       *entry,
 			g_idle_add_full (G_PRIORITY_LOW,
 					 (GSourceFunc) panel_run_dialog_find_command_idle,
 					 dialog, NULL);
+
+	g_free (text);
 }
 
 static void
@@ -1690,69 +1734,74 @@ entry_drag_data_received (GtkEditable      *entry,
 
 static void
 panel_run_dialog_setup_entry (PanelRunDialog *dialog,
-			      GladeXML       *gui)
+			      GtkBuilder     *gui)
 {
 	GdkScreen             *screen;
 	int                    width_request;
+	GtkWidget             *entry;
 	
-	dialog->gnome_entry = glade_xml_get_widget (gui, "gnome_entry");
-	dialog->gtk_entry   = glade_xml_get_widget (gui, "gtk_entry");
+	dialog->combobox = PANEL_GTK_BUILDER_GET (gui, "comboboxentry");
+
+	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
+	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+
+	gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->combobox),
+				 _panel_run_get_recent_programs_list ());
+	gtk_combo_box_entry_set_text_column
+		(GTK_COMBO_BOX_ENTRY (dialog->combobox), 0);
 
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
 
         /* 1/4 the width of the first monitor should be a good value */
 	width_request = panel_multiscreen_width (screen, 0) / 4;
-	g_object_set (G_OBJECT (dialog->gnome_entry),
+	g_object_set (G_OBJECT (dialog->combobox),
 		      "width_request", width_request,
 		      NULL);
 
-        g_signal_connect (dialog->gtk_entry, "key-press-event",
+        g_signal_connect (entry, "key-press-event",
 			  G_CALLBACK (entry_event), dialog);
 			  
-        dialog->changed_id = g_signal_connect (dialog->gtk_entry, "changed",
-					       G_CALLBACK (entry_changed),
+        dialog->changed_id = g_signal_connect (dialog->combobox, "changed",
+					       G_CALLBACK (combobox_changed),
 					       dialog);
 
-	gtk_drag_dest_unset (dialog->gtk_entry);
+	gtk_drag_dest_unset (dialog->combobox);
 	
-	gtk_drag_dest_set (dialog->gtk_entry,
+	gtk_drag_dest_set (dialog->combobox,
 			   GTK_DEST_DEFAULT_MOTION|GTK_DEST_DEFAULT_HIGHLIGHT,
 			   NULL, 0,
 			   GDK_ACTION_COPY);
-	gtk_drag_dest_add_uri_targets (dialog->gtk_entry);
+	gtk_drag_dest_add_uri_targets (dialog->combobox);
 
-	g_signal_connect (dialog->gtk_entry, "drag_data_received",
+	g_signal_connect (dialog->combobox, "drag_data_received",
 			  G_CALLBACK (entry_drag_data_received), dialog);
 }
 
 static char *
 panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 {
-	GKeyFile   *key_file;
-	gboolean    exec = FALSE;
-	const char *text;
-	GFile      *file;
-	char       *name;
-	char       *disk;
-	char       *scheme;
-	char       *save_uri;
+	GKeyFile *key_file;
+	gboolean  exec = FALSE;
+	char     *text;
+	char     *name;
+	char     *disk;
+	char     *scheme;
+	char     *save_uri;
 
-	text = gtk_entry_get_text (GTK_ENTRY (dialog->gtk_entry));
+	text = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox));
 	
-	if (!text || !text [0])
+	if (!text || !text [0]) {
+		g_free (text);
 		return NULL;
+	}
 		
 	key_file = panel_key_file_new_desktop ();
-
 	disk = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
-	file = panel_util_get_file_optional_homedir (disk);
-	g_free (disk);
 
-	scheme = g_file_get_uri_scheme (file);
-	if (!g_ascii_strcasecmp (scheme, "http"))
-		exec = command_is_executable (text);
-	else if (!g_ascii_strcasecmp (scheme, "file"))
-		exec = command_is_executable (text);
+	scheme = g_uri_parse_scheme (disk);
+	/* if it's an absolute path or not a URI, it's possibly an executable */
+	if (g_path_is_absolute (disk) || !scheme)
+		exec = command_is_executable (disk, NULL, NULL);
 	g_free (scheme);
 		
 	if (exec) {
@@ -1760,16 +1809,19 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 		panel_key_file_set_string (key_file, "Exec", text);
 		name = g_strdup (text);
 	} else {
-		char *uri;
+		GFile *file;
+		char  *uri;
 
+		file = panel_util_get_file_optional_homedir (disk);
 		uri = g_file_get_uri (file);
+		g_object_unref (file);
 
 		panel_key_file_set_string (key_file, "Type", "Link");
 		panel_key_file_set_string (key_file, "URL", uri);
 		name = uri;
 	}
 
-	g_object_unref (file);
+	g_free (disk);
 
 	panel_key_file_set_locale_string (key_file, "Name",
 					  (dialog->item_name) ?
@@ -1796,6 +1848,7 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	g_key_file_free (key_file);
 	g_free (disk);
 	g_free (name);
+	g_free (text);
 
 	return save_uri;
 }
@@ -1853,9 +1906,9 @@ panel_run_dialog_screen_changed (GtkWidget      *widget,
 
 static void
 panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
-			       GladeXML       *gui)
+			       GtkBuilder     *gui)
 {
-	dialog->pixmap = glade_xml_get_widget (gui, "icon_pixmap");
+	dialog->pixmap = PANEL_GTK_BUILDER_GET (gui, "icon_pixmap");
 	
 	g_signal_connect (dialog->pixmap, "style-set",
 			  G_CALLBACK (panel_run_dialog_style_set),
@@ -1870,15 +1923,15 @@ panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 }
 
 static PanelRunDialog *
-panel_run_dialog_new (GdkScreen *screen,
-		      GladeXML  *gui,
+panel_run_dialog_new (GdkScreen  *screen,
+		      GtkBuilder *gui,
 		      guint32    activate_time)
 {
 	PanelRunDialog *dialog;
 
 	dialog = g_new0 (PanelRunDialog, 1);
 
-	dialog->run_dialog = glade_xml_get_widget (gui, "panel_run_dialog");
+	dialog->run_dialog = PANEL_GTK_BUILDER_GET (gui, "panel_run_dialog");
 	
 	g_signal_connect_swapped (dialog->run_dialog, "response",
 				  G_CALLBACK (panel_run_dialog_response), dialog);
@@ -1886,8 +1939,8 @@ panel_run_dialog_new (GdkScreen *screen,
 	g_signal_connect_swapped (dialog->run_dialog, "destroy",
 				  G_CALLBACK (panel_run_dialog_destroy), dialog);
 
-	dialog->run_button = glade_xml_get_widget (gui, "run_button");
-	dialog->terminal_checkbox = glade_xml_get_widget (gui, "terminal_checkbox");
+	dialog->run_button = PANEL_GTK_BUILDER_GET (gui, "run_button");
+	dialog->terminal_checkbox = PANEL_GTK_BUILDER_GET (gui, "terminal_checkbox");
 	
 	panel_run_dialog_setup_pixmap        (dialog, gui);
 	panel_run_dialog_setup_entry         (dialog, gui);
@@ -1906,7 +1959,7 @@ panel_run_dialog_new (GdkScreen *screen,
 
 	gtk_window_set_screen (GTK_WINDOW (dialog->run_dialog), screen);
 
-	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_grab_focus (dialog->combobox);
 	gtk_widget_realize (dialog->run_dialog);
 	gdk_x11_window_set_user_time (dialog->run_dialog->window,
 				      activate_time);
@@ -1937,7 +1990,8 @@ void
 panel_run_dialog_present (GdkScreen *screen,
 			  guint32    activate_time)
 {
-	GladeXML *gui;
+	GtkBuilder *gui;
+	GError     *error;
 
 	if (panel_lockdown_get_disable_command_line ())
 		return;
@@ -1946,23 +2000,32 @@ panel_run_dialog_present (GdkScreen *screen,
 		gtk_window_set_screen (GTK_WINDOW (static_dialog->run_dialog), screen);
 		gtk_window_present_with_time (GTK_WINDOW (static_dialog->run_dialog),
 					      activate_time);
-		gtk_widget_grab_focus (static_dialog->gtk_entry);
+		gtk_widget_grab_focus (static_dialog->combobox);
 		return;
 	}
 
-	gui = glade_xml_new (GLADEDIR "/panel-run-dialog.glade",
-			     "panel_run_dialog",
-			     NULL);
-	if (!gui) {
+	gui = gtk_builder_new ();
+	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
+
+	error = NULL;
+	gtk_builder_add_from_file (gui,
+				   BUILDERDIR "/panel-run-dialog.ui",
+				   &error);
+
+        if (error) {
 		char *secondary;
 
-		secondary = g_strdup_printf (_("Unable to load file '%s'."),
-					     GLADEDIR"/panel-run-dialog.glade");
+		secondary = g_strdup_printf (_("Unable to load file '%s': %s."),
+					     BUILDERDIR"/panel-run-dialog.ui",
+					     error->message);
 		panel_error_dialog (NULL, screen, "cannot_display_run_dialog",
 				    TRUE,
 				    _("Could not display run dialog"),
 				      secondary);
 		g_free (secondary);
+		g_error_free (error);
+		g_object_unref (gui);
+
 		return;
 	}
 
