@@ -17,17 +17,19 @@
 #include "panel-util.h"
 
 #include <dirent.h>
+#include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 
+#include <glib.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <gio/gio.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <libgnome/gnome-desktop-item.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-help.h>
-
+#include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-keyfile.h>
 
@@ -39,110 +41,7 @@
 #include "panel-globals.h"
 #include "launcher.h"
 #include "panel-icon-names.h"
-
-static int
-panel_ditem_launch (GnomeDesktopItem  *item,
-		    GList             *file_list,
-		    GdkScreen         *screen,
-		    GError           **error)
-{
-	int workspace;
-
-	workspace = xstuff_get_current_workspace (screen);
-
-	gnome_desktop_item_set_launch_time (item,
-					    gtk_get_current_event_time ());
-
-	return gnome_desktop_item_launch_on_screen (item, file_list, 0,
-						    screen, workspace, error);
-}
-
-void
-panel_util_launch_from_key_file (GKeyFile   *keyfile,
-				 GList      *file_list,
-				 GdkScreen  *screen,
-				 GError    **error)
-{
-	GnomeDesktopItem  *ditem;
-	int                i;
-	static const char *keys [] = { "Type",
-				       "Exec",
-				       "URL",
-				       "Dev",
-				       "Path",
-				       "Terminal",
-				       "TerminalOptions",
-				       "StartupNotify",
-				       "StartupWMClass",
-				     };
-	static const char *locale_keys [] = { "Name",
-					      "GenericName",
-					      "Icon",
-					      "MiniIcon",
-					    };
-
-	g_return_if_fail (keyfile != NULL);
-
-	ditem = gnome_desktop_item_new ();
-	if (ditem == NULL)
-		return;
-
-	for (i = 0; i < G_N_ELEMENTS (keys); i++) {
-		char *value;
-
-		value = panel_key_file_get_string (keyfile, keys [i]);
-		if (value != NULL) {
-			gnome_desktop_item_set_string (ditem, keys [i], value);
-			g_free (value);
-		}
-	}
-
-	for (i = 0; i < G_N_ELEMENTS (locale_keys); i++) {
-		char *value;
-
-		value = panel_key_file_get_locale_string (keyfile,
-							  locale_keys [i]);
-		if (value != NULL) {
-			gnome_desktop_item_set_string (ditem, locale_keys [i], value);
-			g_free (value);
-		}
-	}
-
-	panel_ditem_launch (ditem, file_list, screen, error);
-	gnome_desktop_item_unref (ditem);
-}
-
-void
-panel_launch_desktop_file (const char  *desktop_file,
-			   const char  *fallback_exec,
-			   GdkScreen   *screen,
-			   GError     **error)
-{
-	GnomeDesktopItem *ditem;
-
-	if (g_path_is_absolute (desktop_file))
-		ditem = gnome_desktop_item_new_from_file (desktop_file, 0,
-							  error);
-	else
-		ditem = gnome_desktop_item_new_from_basename (desktop_file, 0,
-							      error);
-
-	if (ditem != NULL) {
-		panel_ditem_launch (ditem, NULL, screen, error);
-		gnome_desktop_item_unref (ditem);
-	} else if (fallback_exec != NULL) {
-		char *argv [2] = {(char *)fallback_exec, NULL};
-
-		if (*error) {
-			g_error_free (*error);
-			*error = NULL;
-		}
-
-		gdk_spawn_on_screen (screen, NULL, argv, NULL,
-				     G_SPAWN_SEARCH_PATH,
-				     NULL, NULL, NULL, error);
-	}
-}
+#include "panel-lockdown.h"
 
 char *
 panel_util_make_exec_uri_for_desktop (const char *exec)
@@ -175,241 +74,6 @@ panel_util_make_exec_uri_for_desktop (const char *exec)
 	return g_string_free (str, FALSE);
 }
 
-void
-panel_show_help (GdkScreen  *screen,
-		 const char *doc_name,
-		 const char *linkid)
-{
-	GError *error = NULL;
-
-	if (!gnome_help_display_desktop_on_screen (NULL, "user-guide", doc_name, linkid, screen, &error)) {
-		panel_error_dialog (NULL, screen, "cannot_show_help", TRUE,
-				    _("Could not display help document"),
-				    error != NULL ? error->message : NULL);
-
-		g_clear_error (&error);
-	}
-}
-
-GList *
-panel_g_list_insert_before (GList *list,
-			    GList *sibling,
-			    GList *link)
-{
-	if (!list) {
-		g_return_val_if_fail (sibling == NULL, list);
-		return link;
-	} else if (sibling) {
-		if (sibling->prev) {
-			link->prev = sibling->prev;
-			link->prev->next = link;
-			link->next = sibling;
-			sibling->prev = link;
-			return list;
-		} else {
-			link->next = sibling;
-			sibling->prev = link;
-			g_return_val_if_fail (sibling == list, link);
-			return link;
-		}
-	} else {
-		GList *last;
-
-		last = list;
-		while (last->next)
-			last = last->next;
-
-		last->next = link;
-		link->prev = last;
-		return list;
-	}
-}
-
-GList *
-panel_g_list_insert_after (GList *list,
-			   GList *sibling,
-			   GList *link)
-{
-	if (!list) {
-		g_return_val_if_fail (sibling == NULL, link);
-		return link;
-	} else if (sibling) {
-		if (sibling->next) {
-			link->next = sibling->next;
-			link->next->prev = link;
-			link->prev = sibling;
-			sibling->next = link;
-			return list;
-		} else {
-			sibling->next = link;
-			link->prev = sibling;
-			return list;
-		}
-			
-	} else {
-		link->next = list;
-		list->prev = link;
-		return link;
-	}
-}
-
-GList *
-panel_g_list_swap_next (GList *list, GList *dl)
-{
-	GList *t;
-
-	if(!dl->next)
-		return list;
-	if(dl->prev)
-		dl->prev->next = dl->next;
-	t = dl->prev;
-	dl->prev = dl->next;
-	dl->next->prev = t;
-	if(dl->next->next)
-		dl->next->next->prev = dl;
-	t = dl->next->next;
-	dl->next->next = dl;
-	dl->next = t;
-
-	if(list == dl)
-		return dl->prev;
-	return list;
-}
-
-GList *
-panel_g_list_swap_prev (GList *list, GList *dl)
-{
-	GList *t;
-
-	if(!dl->prev)
-		return list;
-	if(dl->next)
-		dl->next->prev = dl->prev;
-	t = dl->next;
-	dl->next = dl->prev;
-	dl->prev->next = t;
-	if(dl->prev->prev)
-		dl->prev->prev->next = dl;
-	t = dl->prev->prev;
-	dl->prev->prev = dl;
-	dl->prev = t;
-
-	if(list == dl->next)
-		return dl;
-	return list;
-}
-
-/*maybe this should be a glib function?
- it resorts a single item in the list*/
-GList *
-panel_g_list_resort_item(GList *list, gpointer data, GCompareFunc func)
-{
-	GList *dl;
-
-	if(!list)
-		return NULL;
-
-	dl = g_list_find(list,data);
-
-	g_return_val_if_fail(dl!=NULL,list);
-
-	while(dl->next &&
-	      (*func)(dl->data,dl->next->data)>0)
-		list = panel_g_list_swap_next (list, dl);
-	while(dl->prev &&
-	      (*func)(dl->data,dl->prev->data)<0)
-		list = panel_g_list_swap_prev (list, dl);
-	return list;
-}
-
-GSList *
-panel_g_slist_make_unique (GSList       *list,
-			   GCompareFunc  compare,
-			   gboolean      free_data)
-{
-	GSList *sorted, *l;
-
-	g_assert (compare != NULL);
-
-	if (!list)
-		return NULL;
-
-	sorted = g_slist_copy (list);
-	sorted = g_slist_sort (sorted, compare);
-
-	for (l = sorted; l; l = l->next) {
-		GSList *next;
-
-		next = l->next;
-		if (l->data && next && next->data)
-			if (!compare (l->data, next->data)) {
-				list = g_slist_remove (list, l->data);
-				if (free_data)
-					g_free (l->data);
-			}
-	}
-
-	g_slist_free (sorted);
-
-	return list;
-}
-
-GtkWidget *
-panel_error_dialog (GtkWindow  *parent,
-		    GdkScreen  *screen,
-		    const char *class,
-		    gboolean    auto_destroy,
-		    const char *primary_text,
-		    const char *secondary_text)
-{
-	GtkWidget *dialog;
-	char      *freeme;
-
-	freeme = NULL;
-
-	if (primary_text == NULL) {
-		g_warning ("NULL dialog");
-		 /* No need to translate this, this should NEVER happen */
-		freeme = g_strdup_printf ("Error with displaying error "
-					  "for dialog of class %s", class);
-		primary_text = freeme;
-	}
-
-	dialog = gtk_message_dialog_new (parent, 0, GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK, "%s", primary_text);
-	if (secondary_text != NULL)
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  "%s", secondary_text);
-
-	gtk_widget_add_events (dialog, GDK_KEY_PRESS_MASK);
-	g_signal_connect (dialog, "event",
-			  G_CALLBACK (panel_dialog_window_event), NULL);
-
-	if (screen)
-		gtk_window_set_screen (GTK_WINDOW (dialog), screen);
-
-	if (!parent) {
-		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
-		/* FIXME: We need a title in this case, but we don't know what
-		 * the format should be. Let's put something simple until
-		 * the following bug gets fixed:
-		 * http://bugzilla.gnome.org/show_bug.cgi?id=165132 */
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
-	}
-
-	gtk_widget_show_all (dialog);
-
-	if (auto_destroy)
-		g_signal_connect_swapped (G_OBJECT (dialog), "response",
-					  G_CALLBACK (gtk_widget_destroy),
-					  G_OBJECT (dialog));
-
-	if (freeme)
-		g_free (freeme);
-
-	return dialog;
-}
-
 int
 panel_find_applet_index (GtkWidget *widget)
 {
@@ -502,7 +166,7 @@ panel_ensure_dir (const char *dirname)
 
 	while (p != NULL) {
 		*p = '\0';
-		if (mkdir (parsed, 0700) != 0 &&
+		if (g_mkdir (parsed, 0700) != 0 &&
 		    errno != EEXIST && errno != ENOSYS) {
 			g_free (parsed);
 			return FALSE;
@@ -511,7 +175,7 @@ panel_ensure_dir (const char *dirname)
 		p = strchr (p+1, '/');
 	}
 
-	if (mkdir (parsed, 0700) != 0 &&
+	if (g_mkdir (parsed, 0700) != 0 &&
 	    errno != EEXIST && errno != ENOSYS) {
 		g_free (parsed);
 		return FALSE;
@@ -724,6 +388,10 @@ panel_lock_screen_action_available (const char *action)
 
 	g_return_val_if_fail (action != NULL, FALSE);
 
+	if (strcmp (action, "prefs") != 0 &&
+	    panel_lockdown_get_disable_lock_screen ())
+		return FALSE;
+
 	command = panel_lock_screen_action_get_command (action);
 	if (command)
 		enabled = TRUE;
@@ -743,6 +411,10 @@ panel_lock_screen_action (GdkScreen  *screen,
 	g_return_if_fail (GDK_IS_SCREEN (screen));
 	g_return_if_fail (action != NULL);
 
+	if (strcmp (action, "prefs") != 0 &&
+	    panel_lockdown_get_disable_lock_screen ())
+		return;
+
 	command = panel_lock_screen_action_get_command (action);
 
 	if (!command)
@@ -770,6 +442,106 @@ panel_lock_screen (GdkScreen *screen)
 }
 
 
+#define PANEL_LAUNCHER_PERSONAL_PATH "panel2.d/default/launchers"
+
+static char *
+panel_launcher_get_personal_path (void)
+{
+	return g_build_filename (g_get_home_dir (), ".gnome2",
+				 PANEL_LAUNCHER_PERSONAL_PATH, NULL);
+}
+
+gboolean
+panel_launcher_is_in_personal_path (const char *location)
+{
+	GFile    *file;
+	GFile    *launchers_dir;
+	char     *launchers_path;
+	gboolean  retval;
+
+	if (!location)
+		return FALSE;
+
+	launchers_path = panel_launcher_get_personal_path ();
+	launchers_dir = g_file_new_for_path (launchers_path);
+	g_free (launchers_path);
+
+	file = panel_launcher_get_gfile (location);
+
+	retval = g_file_has_prefix (file, launchers_dir);
+
+	g_object_unref (file);
+	g_object_unref (launchers_dir);
+
+	return retval;
+}
+
+GFile *
+panel_launcher_get_gfile (const char *location)
+{
+	char  *path;
+	GFile *file;
+
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		return g_file_new_for_uri (location);
+
+	if (g_path_is_absolute (location))
+		return g_file_new_for_path (location);
+
+	path = panel_make_full_path (NULL, location);
+	file = g_file_new_for_path (path);
+	g_free (path);
+
+	return file;
+}
+
+char *
+panel_launcher_get_uri (const char *location)
+{
+	char *path;
+	char *uri;
+
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		return g_strdup (location);
+
+	if (!g_path_is_absolute (location))
+		path = panel_make_full_path (NULL, location);
+	else
+		path = g_strdup (location);
+
+	uri = g_filename_to_uri (path, NULL, NULL);
+	g_free (path);
+
+	return uri;
+}
+
+char *
+panel_launcher_get_filename (const char *location)
+{
+	GFile *file;
+	GFile *launchers_dir;
+	char  *launchers_path;
+	char  *retval;
+
+	if (!g_path_is_absolute (location) &&
+	    g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		/* this is not a local URI */
+		return NULL;
+
+	launchers_path = panel_launcher_get_personal_path ();
+	launchers_dir = g_file_new_for_path (launchers_path);
+	g_free (launchers_path);
+
+	file = panel_launcher_get_gfile (location);
+
+	retval = g_file_get_relative_path (launchers_dir, file);
+
+	g_object_unref (file);
+	g_object_unref (launchers_dir);
+
+	return retval;
+}
+
 char *
 panel_make_full_path (const char *dir,
 		      const char *filename)
@@ -780,7 +552,7 @@ panel_make_full_path (const char *dir,
 	g_return_val_if_fail (filename != NULL, NULL);
 
 	if (!dir) {
-		freeme = gnome_util_home_file (PANEL_LAUNCHERS_PATH);
+		freeme = panel_launcher_get_personal_path ();
 		dir = freeme;
 	}
 
@@ -801,7 +573,15 @@ panel_make_unique_desktop_path_from_name (const char *dir,
 {
 	int   num = 1;
 	char *path = NULL;
-	char  filename[MAXNAMLEN];
+#ifndef NAME_MAX
+/* sigh: some OS don't have NAME_MAX (which is POSIX). */
+#ifdef MAXNAMLEN
+#define NAME_MAX MAXNAMLEN
+#else
+#define NAME_MAX 255
+#endif
+#endif
+	char  filename[NAME_MAX];
 
 /* g_file_set_contents() use "%s.XXXXXX"
  * FIXME: waiting for http://bugzilla.gnome.org/show_bug.cgi?id=437977 */
@@ -891,99 +671,6 @@ panel_make_unique_desktop_uri (const char *dir,
 	return uri;
 }
 
-static char *
-lookup_in_data_dir (const char *basename,
-                    const char *data_dir)
-{
-	char *path;
-
-	path = g_build_filename (data_dir, basename, NULL);
-	if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
-		g_free (path);
-		return NULL;
-	}
-
-	return path;
-}
-
-char *
-panel_lookup_in_data_dirs (const char *basename)
-{
-	const char * const *system_data_dirs;
-	const char          *user_data_dir;
-	char                *retval;
-	int                  i;
-
-	user_data_dir    = g_get_user_data_dir ();
-	system_data_dirs = g_get_system_data_dirs ();
-
-	if ((retval = lookup_in_data_dir (basename, user_data_dir)))
-		return retval;
-
-	for (i = 0; system_data_dirs[i]; i++)
-		if ((retval = lookup_in_data_dir (basename, system_data_dirs[i])))
-			return retval;
-
-	return NULL;
-}
-
-/* Stolen from evolution-data-server/libedataserver/e-util.c:
- * e_util_unicode_get_utf8()
- * e_util_utf8_strstrcase() */
-static char *
-panel_util_unicode_get_utf8 (const char *text, gunichar *out)
-{
-	*out = g_utf8_get_char (text);
-	return (*out == (gunichar)-1) ? NULL : g_utf8_next_char (text);
-}
-
-const char *
-panel_util_utf8_strstrcase (const char *haystack, const char *needle)
-{
-	gunichar *nuni;
-	gunichar unival;
-	gint nlen;
-	const char *o, *p;
-
-	if (haystack == NULL) return NULL;
-	if (needle == NULL) return NULL;
-	if (strlen (needle) == 0) return haystack;
-	if (strlen (haystack) == 0) return NULL;
-
-	nuni = g_alloca (sizeof (gunichar) * strlen (needle));
-
-	nlen = 0;
-	for (p = panel_util_unicode_get_utf8 (needle, &unival); p && unival; p = panel_util_unicode_get_utf8 (p, &unival)) {
-		nuni[nlen++] = g_unichar_tolower (unival);
-	}
-	/* NULL means there was illegal utf-8 sequence */
-	if (!p) return NULL;
-
-	o = haystack;
-	for (p = panel_util_unicode_get_utf8 (o, &unival); p && unival; p = panel_util_unicode_get_utf8 (p, &unival)) {
-		gint sc;
-		sc = g_unichar_tolower (unival);
-		/* We have valid stripped char */
-		if (sc == nuni[0]) {
-			const char *q = p;
-			gint npos = 1;
-			while (npos < nlen) {
-				q = panel_util_unicode_get_utf8 (q, &unival);
-				if (!q || !unival) return NULL;
-				sc = g_unichar_tolower (unival);
-				if (sc != nuni[npos]) break;
-				npos++;
-			}
-			if (npos == nlen) {
-				return o;
-			}
-		}
-		o = p;
-	}
-
-	return NULL;
-}
-
 GdkPixbuf *
 panel_util_cairo_rgbdata_to_pixbuf (unsigned char *data,
 				    int            width,
@@ -1084,7 +771,7 @@ panel_util_get_gfile_root (GFile *file)
 	return parent_old;
 }
 
-char *
+static char *
 panel_util_get_icon_name_from_g_icon (GIcon *gicon)
 {
 	const char * const *names;
@@ -1105,97 +792,6 @@ panel_util_get_icon_name_from_g_icon (GIcon *gicon)
 	return NULL;
 }
 
-/* TODO: kill this when we can depend on GTK+ 2.14 */
-static GdkPixbuf *
-panel_util_gdk_pixbuf_load_from_stream (GInputStream  *stream)
-{
-#define LOAD_BUFFER_SIZE 65536
-	unsigned char buffer[LOAD_BUFFER_SIZE];
-	gssize bytes_read;
-	GdkPixbufLoader *loader;
-	GdkPixbuf *pixbuf;
-	gboolean got_eos;
-	
-
-	g_return_val_if_fail (stream != NULL, NULL);
-
-	got_eos = FALSE;
-	loader = gdk_pixbuf_loader_new ();
-	while (1) {
-		bytes_read = g_input_stream_read (stream, buffer, sizeof (buffer),
-						  NULL, NULL);
-		
-		if (bytes_read < 0) {
-			break;
-		}
-		if (bytes_read == 0) {
-			got_eos = TRUE;
-			break;
-		}
-		if (!gdk_pixbuf_loader_write (loader,
-					      buffer,
-					      bytes_read,
-					      NULL)) {
-			break;
-		}
-	}
-
-	g_input_stream_close (stream, NULL, NULL);
-	gdk_pixbuf_loader_close (loader, NULL);
-
-	pixbuf = NULL;
-	if (got_eos) {
-		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-		if (pixbuf != NULL) {
-			g_object_ref (pixbuf);
-		}
-	}
-
-	g_object_unref (loader);
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-panel_util_get_pixbuf_from_g_loadable_icon (GIcon *gicon,
-					    int    size)
-{
-	GdkPixbuf    *pixbuf;
-	GInputStream *stream;
-
-	if (!G_IS_LOADABLE_ICON (gicon))
-		return NULL;
-
-	pixbuf = NULL;
-
-	stream = g_loadable_icon_load (G_LOADABLE_ICON (gicon),
-				       size,
-				       NULL, NULL, NULL);
-	if (stream) {
-		pixbuf = panel_util_gdk_pixbuf_load_from_stream (stream);
-		g_object_unref (stream);
-	}
-
-	if (pixbuf) {
-		gint width, height;
-
-		width = gdk_pixbuf_get_width (pixbuf);
-		height = gdk_pixbuf_get_height (pixbuf);
-
-		if (width > size || height > size) {
-			GdkPixbuf *tmp;
-
-			tmp = gdk_pixbuf_scale_simple (pixbuf, size, size,
-						       GDK_INTERP_BILINEAR);
-
-			g_object_unref (pixbuf);
-			pixbuf = tmp;
-		}
-	}
-
-	return pixbuf;
-}
-
 static char *
 panel_util_get_file_display_name_if_mount (GFile *file)
 {
